
/*
**  jazz - a midi sequencer for Linux
**
**  Copyright (C) 1994 Andreas Voss (andreas@avix.rni.sub.org)
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// 
// $Id: events.cc,v 1.5 1994/06/25 12:24:08 andreas Exp andreas $
// 

#include "events.h"


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdarg.h>



void Error(char *fmt, ...)
{
  char buf[200];
  va_list argptr;
  va_start(argptr, fmt);
  vsprintf(buf, fmt, argptr);
  va_end(argptr);
  fprintf(stderr, "Error %s\n", buf);
  exit(1);
}



int tReadBase::Open(char *fname)
{
  if (fname == NULL)
    fd = stdin;
  else
  {
    fd = fopen(fname, "r");
    if (fd == NULL)
      Error("Fehler beim oeffnen von %s", fname);
  }
  return 1;
}


void tReadBase::Close()
{
  if (fd != stdin)
    fclose(fd);
}




int tWriteBase::Open(char *fname, int nTracks, int TicksPerQuarter)
{
  if (fname == NULL)
    fd = stdout;
  else
  {
    fd = fopen(fname, "w");
    if (fd == NULL)
    {
      Error("Fehler beim oeffnen von %s", fname);
      return 0;
    }
  }
  return nTracks;
}


void tWriteBase::Close()
{
  if (fd != stdout)
    fclose(fd);
}


// -------------------------------------------------------
// tGetMidiBytes
// -------------------------------------------------------

int tGetMidiBytes::Write(tEvent *e, uchar *s, int len)
{
  int Stat = e->Stat;

  switch (Stat)
  {
    case StatKeyOff:
      Stat = StatKeyOn;  // better RunningStatus
    case StatKeyOn:
    case StatAftertouch:
    case StatControl:
    case StatProgram:
    case StatMono:
    case StatPitch:
      nBytes = 0;

      Buffer[nBytes++] = Stat | ((tChannelEvent *)e)->Channel;
      while(len--)
        Buffer[nBytes++] = *s++;
      return 0;

    default:
      return 1;
  }
}

