
/*
**  jazz - a midi sequencer for Linux
**
**  Copyright (C) 1994 Andreas Voss (andreas@avix.rni.sub.org)
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// 
// $Id: med.cc,v 1.6 1994/06/25 12:24:08 andreas Exp andreas $
// 

#ifndef wx_wxh
#include <wx.h>
#endif

#include "song.h"
#include "trackwin.h"
#include "filter.h"

#include "mstdfile.h"
#include "player.h"
#include "jazz.h"
#include <ctype.h>
#include <assert.h>

// -------------------------- global config -----------------------------------

tNamedValue *DrumNames;
tNamedValue *VoiceNames;
tNamedValue *ControlNames;
int DrumChannel;

static void InitConfig(void)
{
  int i;

  DrumChannel = 10;

  DrumNames = new tNamedValue [130];
  for (i = 0; i < 129; i++)
  {
    DrumNames[i].Value = i;
    DrumNames[i].Name  = "";
  }
  DrumNames[129].Name = 0;

  VoiceNames = new tNamedValue [130];
  for (i = 0; i < 129; i++)
  {
    VoiceNames[i].Value = i;
    VoiceNames[i].Name  = "";
  }
  VoiceNames[0].Name = "None";
  VoiceNames[129].Name = 0;

  ControlNames = new tNamedValue [130];
  for (i = 0; i < 129; i++)
  {
    ControlNames[i].Value = i;
    ControlNames[i].Name  = "";
  }
  ControlNames[129].Name = 0;
}



static void LoadConfig(FILE *fd)
{
  char buf[1000];
  int i, j;
  tNamedValue *nv = 0;
  while (fgets(buf, sizeof(buf), fd) != NULL)
  {
    if (strncmp(buf, ".drumchannel", 12) == 0)
      sscanf(buf, ".drumchannel %d", &DrumChannel);
    else if (strncmp(buf, ".drumnames", 10) == 0)
      nv = DrumNames;
    else if (strncmp(buf, ".voicenames", 11) == 0)
      nv = VoiceNames;
    else if (strncmp(buf, ".ctrlnames", 10) == 0)
      nv = ControlNames;
    else if (nv && isdigit(buf[0]))
    {
      sscanf(buf, " %d %n", &i, &j);
      assert(0 <= i && i <= 128);
      buf[ strlen(buf)-1 ] = 0;	// cut off \n
      nv[i+1].Name = strdup(buf + j);
    }
  }
}

// --------------------------- jazz application -------------------------------


class tApp: public wxApp
{
  public:
    wxFrame *OnInit(void);
};


// This statement initialises the whole application
tApp     myApp;



wxFrame *tApp::OnInit(void)
{
  FILE *fd;

  tSong *Song = new tSong;
  Midi = new tMpuPlayer(Song);
  if (!Midi->Installed())
  {
    fprintf(stderr, "MPU-Hardware not found\n");
    Midi = new tNullPlayer(Song);
  }

  TrackWin = new tTrackWin(0, "Jazz!", Song);
  TrackWin->Create();

  InitConfig();
  fd = fopen(".jazz", "r");
  if (!fd)
    fd = fopen("~/.jazz", "r");
  if (fd)
  {
    LoadConfig(fd);
    fclose(fd);
  }

  fd = fopen("jazz.mid", "r");
  if (fd)
  {
    fclose(fd);
    tStdRead io;
    Song->Read(io, "jazz.mid");
  }

  return TrackWin;
}

