
/*
**  jazz - a midi sequencer for Linux
**
**  Copyright (C) 1994 Andreas Voss (andreas@avix.rni.sub.org)
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* 
 * $Id: player.h,v 1.6 1994/06/25 12:24:08 andreas Exp andreas $
 */ 

#ifndef player_h
#define player_h

#ifndef wx_timerh
#include <wx_timer.h>
#endif

#ifndef events_h
#include "events.h"
#endif

#ifndef track_h
#include "track.h"
#endif

#ifndef song_h
#include "song.h"
#endif

#define DB_WRITE 0
#if DB_WRITE
extern int dwrite(int dev, const char *buf, int size);
#else
#define dwrite(a,b,c) write(a,b,c)
#endif

class tPlayer : public wxTimer
{
  protected:

    long OutClock;
    long DeltaClock;

  public:

    int Playing;	// successful StartPlay
    virtual int Installed() = 0;	// Hardware found

    tSong *Song;
    tEventArray PlayBuffer;
    tEventArray RecdBuffer;

    tPlayer(tSong *song);
    virtual ~tPlayer();
    void Notify();
    void FlushToDevice();

    // return 0 = ok, 1 = buffer full, try again later
    virtual int OutEvent(tEvent *e) = 0;
    virtual void OutBreak() = 0;

    // send event immediately ignoring clock
    virtual void OutNow(tEvent *e) = 0;

    // what's played right now?
    virtual long GetRealTimeClock() = 0;

    virtual void StartPlay(long Clock);
    virtual void StopPlay();
    virtual void AllNotesOff(int Reset = 0);
};

extern tPlayer *Midi;

// --------------------------------------------------------
// Roland MPU 401
// --------------------------------------------------------

#include <unistd.h>
#include <fcntl.h>

class tBuffer
{
  public:
    char Buffer[2000];
    int  Written, Read;
    long Clock;
    int  RunningStatus;

    void Clear()
    {
      Read = Written = 0;
      Clock = 0;
      RunningStatus = 0;
    }

    tBuffer()
    {
      Clear();
    }

    int Put(char c)
    {
      if (Written < (int)sizeof(Buffer))
      {
	Buffer[Written++] = c;
        return 0;
      }
      return -1;
    }

    int Get()
    {
      if (Read != Written)
        return (unsigned char)Buffer[Read++];
      return -1;
    }

    int WriteFile(int dev)
    {
      if (Read != Written)
	Read += dwrite(dev, Buffer + Read, Written - Read);
      if (Read != Written)
	return 0;
      Read = Written = 0;
      return 1;
    }

    int ReadFile(int dev)
    {
      int i;
      if (Read)	// move data to beginning of buffer
      {
        for (i = 0; i < Written - Read; i++)
          Buffer[i] = Buffer[i + Read];
        Written -= Read;
        Read = 0;
      }
      Written += read(dev, Buffer + Written, sizeof(Buffer) - Written);
      return Read;
    }
};


// ---------------------------- MPU-Driver -----------------------------

class tMpuPlayer : public tPlayer
{
    int  dev;
    tBuffer PlyBytes;
    tBuffer RecBytes;

  public:

    tMpuPlayer(tSong *song);
    virtual ~tMpuPlayer();
    int  OutEvent(tEvent *e);
    void OutNow(tEvent *e);
    void OutBreak();
    long OutBreak(long Delta);
    void StartPlay(long Clock);
    void StopPlay();
    long GetRealTimeClock();
    int  Installed();
};

#define TRK (0<<6)
#define DAT (1<<6)
#define CMD (2<<6)
#define RES (3<<6)

#define MPUDEVICE "/dev/mpu401"

// ------------------------------ null-driver -------------------------------

class tNullPlayer : public tPlayer
{

  public:

    tNullPlayer(tSong *song) : tPlayer(song) {}
    int Installed() { return 1; }
    virtual ~tNullPlayer() {}
    int  OutEvent(tEvent *e) { return 0; }
    void OutNow(tEvent *e) {}
    void OutBreak() {}
    long OutBreak(long Delta) { return 0; }
    void StartPlay(long Clock) {}
    void StopPlay() {}
    long GetRealTimeClock() { return 0; }
};


#endif
