
/*
**  jazz - a midi sequencer for Linux
**
**  Copyright (C) 1994 Andreas Voss (andreas@avix.rni.sub.org)
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// 
// $Id: trackwin.cc,v 1.6 1994/06/25 12:24:08 andreas Exp andreas $
// 

#include "trackwin.h"
#include "song.h"
#include "mstdfile.h"
#include "filter.h"
#include "pianowin.h"
#include "command.h"
#include "dialogs.h"
#include "player.h"


tTrackWin *TrackWin = 0;


tTrackWin::tTrackWin(wxFrame *frame, char *title, tSong *song)
  : tEventWin(frame, title, song)
{
  NextWin = new tPianoWin(frame, "Piano Roll", Song);
  NextWin->Create();
  nBars = 0;
  RecInfo.Track = 0;
  RecInfo.Muted = 0;

  CounterMode = CmProgram;
  NumberMode = NmMidiChannel;
}


Bool tTrackWin::OnClose()
{
  if (Midi->Playing)
  {
    Midi->StopPlay();
    sleep(1);
  }
  delete Midi;
  delete NextWin;
  return TRUE;
}




void tTrackWin::Setup()
{
  float x, y;

  tEventWin::Setup();

  Canvas->GetTextExtent("H", &x, &y);
  LittleBit = (int)(x/2);

  Canvas->GetTextExtent("HXWjgi", &x, &y);
  hLine = (int)y + LittleBit;
  hTop = hFixedFont + 2 * LittleBit;

  Canvas->GetTextExtent("99", &x, &y);
  wNumber = (int)x + LittleBit;

  Canvas->GetTextExtent("Normal Trackname", &x, &y);
  wName = (int)x + LittleBit;

  Canvas->GetTextExtent("m", &x, &y);
  wState = (int)x + LittleBit;

  Canvas->GetTextExtent("999", &x, &y);
  wPatch = (int)x + 2 * LittleBit;

  wLeft = wNumber + wName + wState + wPatch + 1;

  UnMark();
}



// ************************************************************************
// Menubar
// ************************************************************************

#define MEN_LOAD 	1
#define MEN_SAVE 	2
#define MEN_QUIT 	3
#define MEN_ABOUT 	4
#define MEN_TWSETTING	5
#define MEN_FILTER	6
#define MEN_METERCH	7
#define MEN_MERGE	8
#define MEN_NEW		9
#define MEN_COPY	10
#define MEN_DELETE	11
#define MEN_QUANTIZE	12
#define MEN_UNDO	13
#define MEN_DEBUG	14
#define MEN_SETCHAN	15
#define MEN_TRANSP	16
#define MEN_VELOC	17
#define MEN_LENGTH	18
#define MEN_RESET       19
#define MEN_SONG	20
#define MEN_TMERGE	21
#define MEN_TSPLIT	22


void tTrackWin::CreateMenu()
{
  wxMenuBar *menu_bar = NULL;
  wxMenu *file_menu = new wxMenu;
  file_menu->Append(MEN_LOAD,   "&Load ...");
  file_menu->Append(MEN_MERGE,  "&Merge ...");
  file_menu->Append(MEN_NEW,    "&New ...");
  file_menu->Append(MEN_SAVE,   "&Save ...");
  file_menu->Append(MEN_QUIT,   "&Quit");

  wxMenu *edit_menu = new wxMenu;
  edit_menu->Append(MEN_COPY,      "&Replicate ...");
  edit_menu->Append(MEN_DELETE,    "&Delete ...");
  edit_menu->Append(MEN_QUANTIZE,  "&Quantize ...");
  edit_menu->Append(MEN_SETCHAN,   "&Set MIDI Channel ...");
  edit_menu->Append(MEN_TRANSP,    "&Transpose ...");
  edit_menu->Append(MEN_VELOC,     "&Velocity ...");
  edit_menu->Append(MEN_LENGTH,    "&Length ...");
  // edit_menu->Append(MEN_DEBUG,  "&Debug...");

  wxMenu *setting_menu = new wxMenu;
  setting_menu->Append(MEN_FILTER,    "&Filter ...");
  setting_menu->Append(MEN_TWSETTING, "&Window ...");
  setting_menu->Append(MEN_SONG,      "&Song ...");

  wxMenu *misc_menu = new wxMenu;
  misc_menu->Append(MEN_UNDO,     "&Undo ...");
  misc_menu->Append(MEN_TMERGE,   "&Merge Tracks ...");
  misc_menu->Append(MEN_TSPLIT,   "&Split Tracks ...");
  misc_menu->Append(MEN_METERCH,  "&Meterchange ...");
  misc_menu->Append(MEN_RESET,    "&Reset Midi");

  wxMenu *help_menu = new wxMenu;
  help_menu->Append(MEN_ABOUT, "&About");

  menu_bar = new wxMenuBar;
  menu_bar->Append(file_menu,    "&File");
  menu_bar->Append(edit_menu,    "&Edit");
  menu_bar->Append(setting_menu, "&Settings");
  menu_bar->Append(misc_menu,    "&Misc");
  menu_bar->Append(help_menu,    "&About");

  SetMenuBar(menu_bar);
}



void tTrackWin::OnMenuCommand(int id)
{
  char *s;
  switch (id)
  {
    case MEN_MERGE:
      s = wxFileSelector("Merge File", NULL, NULL, NULL, "*.mid");
      if (s)
      {
        tStdRead io;
        Song->Read(io, s);
        Redraw();
      }
      break;

    case MEN_NEW:
      if (wxMessageBox("Clear Song?", "Shure?", wxOK | wxCANCEL) == wxOK)
      {
	Song->Clear();
	Redraw();
      }
      break;

    case MEN_LOAD:
      s = wxFileSelector("Load File", NULL, NULL, NULL, "*.mid");
      if (s)
      {
        tStdRead io;
        Song->Clear();
        Song->Read(io, s);
        SetTitle(s);
        Redraw();
      }
      break;

    case MEN_SAVE:
      s = wxFileSelector("Save File", NULL, NULL, NULL, "*.mid");
      if (s)
      {
        tStdWrite io;
        Song->Write(io, s);
      }
      break;

    case MEN_QUIT:
      OnClose();
      delete this;
      break;

    case MEN_TWSETTING: SettingsDialog(0); break;
    case MEN_COPY: 	MenCopy(); break;
    case MEN_QUANTIZE: 	MenQuantize(); break;
    case MEN_DELETE: 	MenDelete(); break;
    case MEN_SETCHAN: 	MenSetChannel(); break;
    case MEN_TRANSP: 	MenTranspose(); break;
    case MEN_VELOC: 	MenVelocity(); break;
    case MEN_LENGTH: 	MenLength(); break;
    case MEN_METERCH:	MenMeterChange(); break;
    case MEN_RESET:	Midi->AllNotesOff(1); break;

    case MEN_SONG: 	MenSongSettings(); break;
    case MEN_TSPLIT: 	MenSplitTracks(); break;
    case MEN_TMERGE: 	MenMergeTracks(); break;

    case MEN_UNDO:
      Song->Undo();
      Redraw();
      break;

    case MEN_DEBUG:
            break;

    case MEN_FILTER:
      Filter->Dialog(0);
      break;

    case MEN_ABOUT:
      wxMessageBox(

"jazz, version 1.0\n\
\n\
Copyright (C) 1994 Andreas Voss\n\
\n\
This program is free software WITHOUT ANY WARRANTY; you can redistribute\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n\
\n\
feedback welcome to andreas@avix.rni.sub.org",
"About");

      break;
  }
}


// ************************************************************************
// Painting
// ************************************************************************



void tTrackWin::DrawCounters()
{
  char *str;
  int i;
  switch (CounterMode)
  {
    case CmProgram: str = "Prg"; break;
    case CmVolume : str = "Vol"; break;
    case CmPan    : str = "Pan"; break;
    case CmReverb : str = "Rev"; break;
    case CmChorus : str = "Cho"; break;
    default       : str = "???"; break;
  }
  LineText(xPatch, yy-1, wPatch, str, hTop);

  dc->SetClippingRegion(xPatch, yEvents, xPatch + wPatch, yEvents + hEvents);
  for (i = FromLine; i < ToLine; i++)
  {
    tTrack *t = Song->GetTrack(i);
    if (t)
    {
      char buf[20];
      int  val;
      switch (CounterMode)
      {
        case CmProgram: val = t->GetPatch(); break;
        case CmVolume : val = t->GetVolume(); break;
        case CmPan    : val = t->GetPan(); break;
        case CmReverb : val = t->GetReverb(); break;
        case CmChorus : val = t->GetChorus(); break;
        default : val = 0; break;
      }
      sprintf(buf, "%3d", val);
      LineText(xPatch, Line2y(i), wPatch, buf);
    }
    else
      LineText(xPatch, Line2y(i), wPatch, "?");
  }
  dc->DestroyClippingRegion();
}




void tTrackWin::DrawNumbers()
{
  char *str;
  int i;
  switch (NumberMode)
  {
    case NmTrackNr     : str = "T"; break;
    case NmMidiChannel : str = "M"; break;
    default            : str = "?"; break;
  }
  LineText(xNumber, yy-1, wNumber, str, hTop);

  dc->SetClippingRegion(xNumber, yEvents, xNumber + wNumber, yEvents + hEvents);
  for (i = FromLine; i < ToLine; i++)
  {
    tTrack *t = Song->GetTrack(i);
    if (t)
    {
      char buf[20];
      int  val;
      switch (NumberMode)
      {
        case NmTrackNr: val = i; break;
        case NmMidiChannel : val = t->Channel; break;
        default : val = 0; break;
      }
      sprintf(buf, "%02d", val);
      LineText(xNumber, Line2y(i), wNumber, buf);
    }
    else
      LineText(xNumber, Line2y(i), wNumber, "?");
  }
  dc->DestroyClippingRegion();
}



void tTrackWin::DrawSpeed(int Value)
{
  char buf[50];
  if (Value < 0)
    Value = Song->GetTrack(0)->GetSpeed();
  sprintf(buf, "speed: %3d", Value);
  LineText(xName, yy-1, wName, buf, hTop);
}


void tTrackWin::OnPaint(int What, long x, long y, long w, long h)
{
  tEventWin::OnPaint(What, x, y, w, h);

  xNumber  = xx;
  xName    = xNumber  + wNumber;
  xState   = xName    + wName;
  xPatch   = xState   + wState;

  long StopClk;
  tBarInfo BarInfo(Song);
  char buf[20];

  dc->Clear();

  #define VLine(x) DrawLine(x,  yy, x, yEvents+hEvents)
  #define HLine(y) DrawLine(xx, y, xx + ww, y)

  dc->SetPen(wxBLACK_PEN);

  // vertikale Linien
  dc->VLine(xNumber);
  dc->VLine(xName);
  dc->VLine(xState);
  dc->VLine(xPatch);
  dc->VLine(xEvents);
  dc->VLine(xEvents-1);
  dc->HLine(yEvents);
  dc->HLine(yEvents-1);


  // Taktstriche und -nummern

  BarInfo.SetClock(FromClock);
  StopClk = x2Clock(xx + ww);
  nBars = 0;
  dc->SetPen(wxBLACK_DASHED_PEN);
  while (1)
  {
    x = Clock2x(BarInfo.Clock);
    if (x > xx + ww)
      break;
    if (x >= xEvents)	// so ne Art clipping
    {
      if ((BarInfo.BarNr % 4) == 0)
      {
	dc->SetPen(wxBLACK_PEN);
	sprintf(buf, "%d", BarInfo.BarNr + 1);
	dc->DrawText(buf, x + LittleBit, yEvents - hLine);
	dc->DrawLine(x, yEvents - hLine, x, yEvents + hEvents);
	dc->SetPen(wxBLACK_DASHED_PEN);
      }
      else
	dc->DrawLine(x, yEvents, x, yEvents+hEvents);

      if (nBars < MaxBars)	// x-Koordinate fuer MouseAction->Snap()
	xBars[nBars++] = x;

    }
    BarInfo.Next();
  }
  dc->SetPen(wxBLACK_PEN);


  DrawNumbers();
  DrawSpeed();
  DrawCounters();


  // je Track Nummer, Name, State, Patch anzeigen

  dc->SetClippingRegion(xx, yEvents, xx+ww, yEvents + hEvents);
  int TrackNr = FromLine;
  for (y = Line2y(TrackNr); y < yEvents + hEvents; y += hLine)
  {
    dc->HLine(y);
    tTrack *Track = Song->GetTrack(TrackNr);
    if (Track)
    {


      // TrackName
      dc->DrawText(Track->GetName(), xName + LittleBit, y + LittleBit);

      // TrackStatus
      dc->DrawText(Track->GetStateChar(), xState + LittleBit, y + LittleBit);


    }
    ++ TrackNr;
  }
  dc->DestroyClippingRegion();


  // Events zeichnen
  dc->SetClippingRegion(xEvents, yEvents, wEvents, hEvents);
  TrackNr = FromLine;
  for (y = Line2y(TrackNr); y < yEvents + hEvents; y += hLine)
  {
    tTrack *Track = Song->GetTrack(TrackNr);
    if (Track)
    {
      tEventIterator Iterator(Track);
      tEvent *e = Iterator.Range(FromClock, StopClk);
      float y0 = y + LittleBit;
      float y1 = y + hLine - LittleBit;

      if (UseColors)
      {
	while (e)	// slow!
	{
	  float x = Clock2x(e->Clock);
	  dc->SetPen(e->GetPen());
	  dc->DrawLine(x, y0, x, y1);
	  e = Iterator.Next();
	}
	dc->SetPen(wxBLACK_PEN);
      }
      else
      {
	while (e)
	{
	  float x = Clock2x(e->Clock);
	  dc->DrawLine(x, y0, x, y1);
	  e = Iterator.Next();
	}
      }
    }

    ++ TrackNr;
  }

  SnapSel->Draw(xEvents, yEvents, wEvents, hEvents);

  if (Marked.x > 0)
    LineText((long)Marked.x, (long)Marked.y, (long)Marked.w, ">");

  dc->DestroyClippingRegion();
  DrawPlayPosition();

}

// ************************************************************************
// Utilities
// ************************************************************************

long tTrackWin::x2xBar(long x)
{
  for (int i = 1; i < nBars; i++)
    if (x < xBars[i])
      return xBars[i - 1];
  return -1;
}


long tTrackWin::x2wBar(long x)
{
  for (int i = 1; i < nBars; i++)
    if (x < xBars[i])
      return xBars[i] - xBars[i - 1];
  return 0;
}


tTrack *tTrackWin::y2Track(long y)
{
  return Song->GetTrack(y2Line(y));
}

void tTrackWin::Mark(long x, long y)
{
  Marked.x = x2xBar(x);
  Marked.y = y2yLine(y);
  Marked.w = x2wBar(x);
  Marked.h = hLine;
  LineText((int)Marked.x, (int)Marked.y, (int)Marked.w, ">");
}

void tTrackWin::UnMark()
{
  Marked.x = -1;
}


// ********************************************************************
// PlayPosition
// ********************************************************************

void tTrackWin::NewPlayPosition(long Clock)
{
  if (!SnapSel->Active && Clock > (FromClock + 5 * ToClock) / 6L)
  {
    long x = Clock2x(Clock);
    Canvas->SetScrollPosition(x - wLeft, yy);
  }
  tEventWin::NewPlayPosition(Clock);
}

// ********************************************************************
// Snapper
// ********************************************************************

void tTrackWin::SnapSelStart(wxMouseEvent &e)
{
  SnapSel->SetXSnap(nBars, xBars);
  SnapSel->SetYSnap(Line2y(FromLine), yEvents + hEvents, hLine);
}


void tTrackWin::SnapSelStop(wxMouseEvent &e)
{
  if (SnapSel->Selected)
  {
    Filter->FromTrack = y2Line((long)SnapSel->r.y);
    Filter->ToTrack   = y2Line((long)(SnapSel->r.y + SnapSel->r.h - 1));
    Filter->FromClock = x2BarClock((long)SnapSel->r.x + 1);
    Filter->ToClock   = x2BarClock((long)(SnapSel->r.x + SnapSel->r.w + 1));
  }
}



// --------------------------------------------------------------------
// Tracknummer
// --------------------------------------------------------------------

void tTrackWin::MouseNumber(wxMouseEvent &e)
{
}

// --------------------------------------------------------------------
// PatchCounter
// --------------------------------------------------------------------

class tPatchCounter : public tMouseCounter
{
    tTrackWin *tw;
  public:
    int Event(wxMouseEvent &e);
    tPatchCounter(tTrackWin *t, tRect *r, int v, int min, int max)
      : tMouseCounter(t->dc, r, v, min, max)
    { 
      tw = t; 
    }
};


int tPatchCounter::Event(wxMouseEvent &e)
{
  if (tMouseCounter::Event(e))
  {
    tTrack *t = tw->y2Track((long)r.y);
    if (t)
    {
      switch (tw->CounterMode)
      {
        case CmProgram : t->SetPatch(Value); break;
        case CmVolume  : t->SetVolume(Value); break;
        case CmPan     : t->SetPan(Value); break;
        case CmReverb  : t->SetReverb(Value); break;
        case CmChorus  : t->SetChorus(Value); break;
        default: break;
      }
    }
    tw->MouseAction = 0;
    delete this;
  }
  return 0;
}


void tTrackWin::MousePatch(wxMouseEvent &e)
{
  float x, y;

  if (!e.LeftDown() && !e.RightDown())
    return;

  e.Position(&x, &y);
  tTrack *t = y2Track((long)y);
  if (t)
  {
    tRect r;
    int Value;
    switch (CounterMode)
    {
      case CmProgram: Value = t->GetPatch(); break;
      case CmVolume : Value = t->GetVolume(); break;
      case CmPan    : Value = t->GetPan(); break;
      case CmReverb : Value = t->GetReverb(); break;
      case CmChorus : Value = t->GetChorus(); break;
      default       : Value = 0; break;
    }
    r.x = xPatch + LittleBit;
    r.y = y2yLine((long)y) + LittleBit;
    r.w = wPatch - LittleBit;
    r.h = hLine - LittleBit;

    tPatchCounter *PatchCounter = new tPatchCounter(this, &r, Value, 0, 128);
    PatchCounter->Event(e);
    MouseAction = PatchCounter;
  }
}

// --------------------------------------------------------------------
// SpeedCounter
// --------------------------------------------------------------------

class tSpeedCounter : public tMouseCounter
{
    tTrackWin *tw;
  public:
    int Event(wxMouseEvent &e);
    tSpeedCounter(tTrackWin *t, tRect *r, int v, int min, int max)
      : tMouseCounter(t->dc, r, v, min, max)
    { 
      tw = t; 
    }
    void ShowValue();
};


int tSpeedCounter::Event(wxMouseEvent &e)
{
  if (tMouseCounter::Event(e))
  {
    tTrack *t = tw->Song->GetTrack(0);
    t->SetSpeed(Value);
    tw->MouseAction = 0;
    delete this;
  }
  return 0;
}

void tSpeedCounter::ShowValue()
{
  tw->DrawSpeed(Value);
}

void tTrackWin::MouseSpeed(wxMouseEvent &e)
{
  tRect r;
  int Value;

  if (!e.LeftDown() && !e.RightDown())
    return;

  Value = Song->GetTrack(0)->GetSpeed();
  tSpeedCounter *SpeedCounter = new tSpeedCounter(this, &r, Value, 20, 250);
  SpeedCounter->Event(e);
  MouseAction = SpeedCounter;
}

// -----------------------------------------------------------------------
// Track-Status
// -----------------------------------------------------------------------


void tTrackWin::MouseState(wxMouseEvent &e)
{
  if (e.LeftDown() || e.RightDown())
  {
    float x, y;
    e.Position(&x, &y);
    tTrack *t = y2Track((long)y);
    if (t)
    {
      t->ToggleState(e.LeftDown() ? 1 : -1); // toggle
      LineText(xState, (long)y, wState, t->GetStateChar());
    }
  }
}


// --------------------------------------------------------------------------
// Name
// --------------------------------------------------------------------------

void tTrackWin::MouseName(wxMouseEvent &e)
{
  if (e.LeftDown())
  {
    float x, y;
    e.Position(&x, &y);
    tTrack *t = y2Track((long)y);
    if (t)
      t->Dialog(this);
  }
}

// --------------------------------------------------------------------------
// Event-Bereich
// --------------------------------------------------------------------------

void tTrackWin::MouseEvents(wxMouseEvent &e)
{
  if (e.RightDown())
  {
    float x, y;
    e.Position(&x, &y);
    int TrackNr = y2Line((long)y);
    long Clock  = x2BarClock((long)x);
    NextWin->NewPosition(TrackNr, Clock);
  }
  else
    tEventWin::OnMouseEvent(e);
}

// --------------------------------------------------------------------------
// Playbar
// --------------------------------------------------------------------------

/*
 * not playing:
 *  events selected:
 *    left : start rec/play
 *    right: mute + start rec/play
 *  no events selected:
 *    left+right: start play
 * playing:
 *  left+right: stop
 */

void tTrackWin::MousePlay(wxMouseEvent &e)
{
  if (e.LeftDown() || e.RightDown())
  {
    float x, y;
    e.Position(&x, &y);

    if (!Midi->Playing)		// Start Record/Play
    {
      if (SnapSel->Selected)
      {
        RecInfo.TrackNr   = Filter->FromTrack;
	RecInfo.Track     = Song->GetTrack(RecInfo.TrackNr);
	RecInfo.FromClock = Filter->FromClock;
	RecInfo.ToClock   = Filter->ToClock;
	if (e.RightDown())
	{
	  RecInfo.Muted = 1;
	  RecInfo.Track->SetState(tsMute);
	  LineText(xState, Line2y(RecInfo.TrackNr), wState, RecInfo.Track->GetStateChar());
	}
	else
	  RecInfo.Muted = 0;
      }
      else
	RecInfo.Track = 0;

      long Clock = x2BarClock((long)x);
      Midi->StartPlay(Clock);
    }

    else			// Stop Record/Play
    {
      Midi->StopPlay();
      if (RecInfo.Track)
      {
        if (RecInfo.Muted)
        {
	  RecInfo.Track->SetState(tsPlay);
	  LineText(xState, Line2y(RecInfo.TrackNr), wState, RecInfo.Track->GetStateChar());
	}
	if (!Midi->RecdBuffer.IsEmpty())
	{
	  int choice = wxMessageBox("Keep recorded events?", "You played", wxOK | wxCANCEL);
	  if (choice == wxOK)
	  {
	    Song->NewUndoBuffer();
	    RecInfo.Track->MergeRange(&Midi->RecdBuffer, RecInfo.FromClock, RecInfo.ToClock, RecInfo.Muted);
	  }
	}
      }
    }
  }
}



// -----------------------------------------------------------------------
// Event-Verteiler 
// -----------------------------------------------------------------------


int tTrackWin::OnMouseEvent(wxMouseEvent &e)
{

  if (!MouseAction)
  {
    float x, y;
    e.Position(&x, &y);
    if (y > yEvents)
    {
      // Events
      if (xNumber < x && x < xNumber + wNumber)
        MouseNumber(e);
      else if (xName < x && x < xName + wName)
	MouseName(e);
      else if(xState < x && x < xState + wState)
	MouseState(e);
      else if (xPatch < x && x < xPatch + wPatch)
	MousePatch(e);
      else if (xEvents < x && x < xEvents + wEvents)
	MouseEvents(e);
      else 
        tEventWin::OnMouseEvent(e);
    }
    else
    {
      // Playbar

      if (xNumber < x && x < xNumber + wNumber)
      {
        if (e.LeftDown())
        {
          NumberMode = (tNumberModes)(((int)NumberMode + 1) % (int)NmModes);
          DrawNumbers();
	}
      }
      else if (xName < x && x < xName + wName)
	MouseSpeed(e);
      else if(xState < x && x < xState + wState)
	;
      else if (xPatch < x && x < xPatch + wPatch)
      {
        if (e.LeftDown())
        {
          CounterMode = (tCounterModes)(((int)CounterMode + 1) % (int)CmModes);
	  DrawCounters();
	}
        else if (e.RightDown())
        {
          CounterMode = (tCounterModes)(((int)CounterMode + (int)CmModes - 1) % (int)CmModes);
	  DrawCounters();
	}
      }
      else if (xEvents < x && x < xEvents + wEvents)
	MousePlay(e);
      else 
        tEventWin::OnMouseEvent(e);
    }
  }

  else
    tEventWin::OnMouseEvent(e);

  return 0;
}



// **************************************************************************
// Copy
// **************************************************************************


class tCopyDlg;

class tCopyCommand : public wxObject, public tMouseAction
{
    tTrackWin *tw;
    int MarkRepeat;
    float StartX, StartY, StopX, StopY;

    tMarkDestin *Mouse;
    tCopyDlg *CopyDlg;

  public:
    static Bool RepeatCopy;
    static Bool EraseSource;
    static Bool EraseDestin;
    static Bool InsertSpace;

    int Event(wxMouseEvent &e);
    tCopyCommand(tTrackWin *t);
    void EditForm(wxPanel *panel);
    void OnOk();
    void OnCancel();
    void Execute(int doit);
};    

Bool tCopyCommand::EraseDestin = 1;
Bool tCopyCommand::RepeatCopy = 0;
Bool tCopyCommand::EraseSource = 0;
Bool tCopyCommand::InsertSpace = 0;


class tCopyDlg : public wxForm
{
  tCopyCommand *cp;

 public:
  tCopyDlg::tCopyDlg(tCopyCommand *c)   { cp = c; }
  void OnOk()				{ cp->OnOk(); wxForm::OnOk(); }
  void OnCancel()			{ cp->OnCancel(); wxForm::OnCancel(); }
  void EditForm(wxPanel *panel);
};


void tCopyDlg::EditForm(wxPanel *panel)
{
  Add(wxMakeFormBool("Erase Destin", &cp->EraseDestin));
  Add(wxMakeFormBool("Repeat Copy", &cp->RepeatCopy));
  Add(wxMakeFormBool("Erase Source", &cp->EraseSource));
  Add(wxMakeFormBool("Insert Space", &cp->InsertSpace));
  AssociatePanel(panel);
}



tCopyCommand::tCopyCommand(tTrackWin *t)
{
  tw = t;
  MarkRepeat = 0;
  Mouse = new tMarkDestin(tw->Canvas, tw, 0);
  CopyDlg = 0;
}


int tCopyCommand::Event(wxMouseEvent &e)
{
  if (!CopyDlg && Mouse && Mouse->Event(e))
  {
    if (Mouse->Aborted)
    {
      if (CopyDlg)
        CopyDlg->OnCancel();
      else
        Execute(0);
    }
    else if (!MarkRepeat)
    {
      e.Position(&StartX, &StartY);
      tw->Mark((long)StartX, (long)StartY);
      wxDialogBox *panel = new wxDialogBox(tw, "Replicate", FALSE);
      CopyDlg = new tCopyDlg(this);
      CopyDlg->EditForm(panel);
      panel->Fit();
      panel->Show(TRUE);
    }
    else
    {
      e.Position(&StopX, &StopY);
      Execute(1);
    }
  }
  return 0;
}


void tCopyCommand::OnOk()
{
  CopyDlg = 0;
  if (RepeatCopy)
  {
    delete Mouse;
    Mouse = new tMarkDestin(tw->Canvas, tw, 1);
    MarkRepeat = 1;
  }
  else
    Execute(1);
}


void tCopyCommand::OnCancel()
{
  CopyDlg = 0;
  Execute(0);
}


void tCopyCommand::Execute(int doit)
{

  if (doit)
  {
    long DestTrack = tw->y2Line((long)StartY);
    long DestClock = tw->x2BarClock((long)StartX);
    tCmdCopy cpy(tw->Filter, DestTrack, DestClock);

    if (RepeatCopy)
      cpy.RepeatClock = tw->x2BarClock((long)StopX, 1);
    cpy.EraseSource = EraseSource;
    cpy.EraseDestin = EraseDestin;
    cpy.InsertSpace = InsertSpace;
    cpy.Execute();
  }

  tw->UnMark();
  tw->MouseAction = 0;
  tw->Redraw(-1);
  delete Mouse;
  delete this;
}


void tTrackWin::MenCopy()
{
  if (!EventsSelected())
    return;
  MouseAction = new tCopyCommand(this);
}

// ******************************************************************
// Song-Settings Dialog
// ******************************************************************



class tSongSettingsDlg : public wxForm
{
 public:
  tEventWin *EventWin;
  int TicksPerQuarter;
  tSongSettingsDlg(tEventWin *w);
  void EditForm(wxPanel *panel);
  virtual void OnOk();
  virtual void OnCancel();
};



tSongSettingsDlg::tSongSettingsDlg(tEventWin *w)
{
  EventWin = w;
  TicksPerQuarter = EventWin->Song->TicksPerQuarter;
}



void tSongSettingsDlg::OnCancel()
{
  EventWin->DialogBox = 0;
  wxForm::OnCancel();
}


void tSongSettingsDlg::OnOk()
{
  EventWin->Song->SetTicksPerQuarter(TicksPerQuarter);
  EventWin->Redraw();
  EventWin->DialogBox = 0;
  wxForm::OnOk();
}



void tSongSettingsDlg::EditForm(wxPanel *panel)
{
  Add(wxMakeFormMessage("Supported: 48, 72, 96, 120, 144, 168, 192"));
  Add(wxMakeFormNewLine());
  Add(wxMakeFormShort("Ticks per Quarter:", &TicksPerQuarter, wxFORM_DEFAULT,
                       new wxList(wxMakeConstraintRange(48.0, 192), 0)));
  AssociatePanel(panel);
}


void tTrackWin::MenSongSettings()
{
  tSongSettingsDlg *dlg;
  if (DialogBox)
  {
    DialogBox->Show(TRUE);
    return;
  }
  DialogBox = new wxDialogBox(this, "Song Settings", FALSE);
  dlg = new tSongSettingsDlg(this);
  dlg->EditForm(DialogBox);
  DialogBox->Fit();
  DialogBox->Show(TRUE);
}

// *************************************************************************************
// Split / Merge Tracks
// *************************************************************************************

void tTrackWin::MenMergeTracks()
{
  int choice = wxMessageBox("Merge all Tracks to Track 0?", "Shure?", wxOK | wxCANCEL);
  if (choice == wxOK)
  {
    int tn;
    Song->NewUndoBuffer();
    tTrack *dst = Song->GetTrack(0);
    for (tn = 1; tn < Song->nTracks; tn++)
    {
      tTrack *src = Song->GetTrack(tn);
      tEventIterator Iterator(src);
      tEvent *e = Iterator.First();
      while (e)
      {
        tEvent *c = e->Copy();
        src->Kill(e);
        dst->Put(c);
        e = Iterator.Next();
      }
      src->Cleanup();
    }
    dst->Cleanup();
  }
}


void tTrackWin::MenSplitTracks()
{
  int choice = wxMessageBox("Split Track 0 by Midi-Channel to Track 1..16?", "Shure?", wxOK | wxCANCEL);
  if (choice == wxOK)
  {
    int ch;
    Song->NewUndoBuffer();
    tTrack *src = Song->GetTrack(0);
    tEventIterator Iterator(src);
    tEvent *e = Iterator.First();
    while (e)
    {
      tChannelEvent *ce = e->IsChannelEvent();
      if (ce)
      {
	int cn = ce->Channel;
	tTrack *dst = Song->GetTrack(cn + 1);
	if (dst)
	{
	  tEvent *cp = ce->Copy();
	  src->Kill(ce);
	  dst->Put(cp);
	}
      }
      e = Iterator.Next();
    }

    for (ch = 0; ch <= 16; ch++)
    {
      src = Song->GetTrack(ch);
      if (src)
        src->Cleanup();
    }
  }
}

